@@ -30,15 +30,14 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
-import java.util.jar.JarEntry;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import java.util.zip.CRC32;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipException;
-import java.util.zip.ZipInputStream;
-import java.util.zip.ZipOutputStream;
 
 import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
+import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
 import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
 import org.apache.commons.compress.archivers.zip.ZipFile;
 import org.apache.commons.compress.utils.SeekableInMemoryByteChannel;
@@ -46,6 +45,9 @@ import org.apache.commons.io.IOUtils;
 import org.apache.commons.io.input.CloseShieldInputStream;
 import org.apache.commons.io.output.CloseShieldOutputStream;
 
+/**
+ * The main class for the Migration tool.
+ */
 public class Migration {
 
     private static final Logger logger = Logger.getLogger(Migration.class.getCanonicalName());
@@ -89,13 +91,20 @@ public class Migration {
         DEFAULT_EXCLUDES.add("spring-security-rsa-*.jar");
     }
 
-    private EESpecProfile profile = EESpecProfile.TOMCAT;
+    private EESpecProfile profile = EESpecProfiles.TOMCAT;
+
+    private boolean enableDefaultExcludes = true;
     private boolean zipInMemory;
+    private boolean converted;
+    private State state = State.NOT_STARTED;
     private File source;
     private File destination;
     private final List<Converter> converters;
     private final Set<String> excludes = new HashSet<>();
 
+    /**
+     * Construct a new migration tool instance.
+     */
     public Migration() {
         // Initialise the converters
         converters = new ArrayList<>();
@@ -108,6 +117,24 @@ public class Migration {
         converters.add(new PassThroughConverter());
     }
 
+    /**
+     * The tool state.
+     */
+    public enum State {
+        /**
+         * Migration not started yet.
+         */
+        NOT_STARTED,
+        /**
+         * Migration in progress.
+         */
+        RUNNING,
+        /**
+         * Migration complete.
+         */
+        COMPLETE
+    }
+
     /**
      * Set the Jakarta EE specifications that should be used.
      *
@@ -126,14 +153,34 @@ public class Migration {
         return profile;
     }
 
+    /**
+     * Enable the default exclusion list for the tool.
+     * @param enableDefaultExcludes true to enable the default
+     */
+    public void setEnableDefaultExcludes(boolean enableDefaultExcludes) {
+        this.enableDefaultExcludes = enableDefaultExcludes;
+    }
+
+    /**
+     * Buffer all conversion operations for compressed archives in memory.
+     * @param zipInMemory true to buffer in memory
+     */
     public void setZipInMemory(boolean zipInMemory) {
         this.zipInMemory = zipInMemory;
     }
 
+    /**
+     * Add specified resource exclusion.
+     * @param exclude the exclude to add
+     */
     public void addExclude(String exclude) {
         this.excludes.add(exclude);
     }
 
+    /**
+     * Set source file.
+     * @param source the source file
+     */
     public void setSource(File source) {
         if (!source.canRead()) {
             throw new IllegalArgumentException(sm.getString("migration.cannotReadSource",
@@ -142,13 +189,40 @@ public class Migration {
         this.source = source;
     }
 
-
+    /**
+     * Set destination file.
+     * @param destination the destination file
+     */
     public void setDestination(File destination) {
         this.destination = destination;
     }
 
 
+    /**
+     * <b>NOTE</b>:
+     * this method is not to indicate that no changes were made,
+     * but that the source can be used and satisfy the selected profile.
+     * @return true if converted occurs
+     */
+    public boolean hasConverted() {
+        if (state != State.COMPLETE) {
+            throw new IllegalStateException(sm.getString("migration.notCompleted"));
+        }
+        return converted;
+    }
+
+
+    /**
+     * Execute migration operation.
+     * @throws IOException when an exception occurs
+     */
     public void execute() throws IOException {
+        if (state == State.RUNNING) {
+            throw new IllegalStateException(sm.getString("migration.alreadyRunning"));
+        }
+        state = State.RUNNING;
+        converted = false;
+
         logger.log(Level.INFO, sm.getString("migration.execute", source.getAbsolutePath(),
                 destination.getAbsolutePath(), profile.toString()));
 
@@ -160,7 +234,7 @@ public class Migration {
                 throw new IOException(sm.getString("migration.mkdirError", destination.getAbsolutePath()));
             }
         } else {
-            // Single file
+            // Single file`
             File parentDestination = destination.getAbsoluteFile().getParentFile();
             if (parentDestination.exists() || parentDestination.mkdirs()) {
                 migrateFile(source, destination);
@@ -168,11 +242,11 @@ public class Migration {
                 throw new IOException(sm.getString("migration.mkdirError", parentDestination.getAbsolutePath()));
             }
         }
+        state = State.COMPLETE;
         logger.log(Level.INFO, sm.getString("migration.done",
                 Long.valueOf(TimeUnit.MILLISECONDS.convert(System.nanoTime() - t1, TimeUnit.NANOSECONDS))));
     }
 
-
     private void migrateDirectory(File src, File dest) throws IOException {
         // Won't return null because src is known to be a directory
         String[] files = src.list();
@@ -196,7 +270,7 @@ public class Migration {
         boolean inplace = src.equals(dest);
         if (!inplace) {
             try (InputStream is = new FileInputStream(src);
-                 OutputStream os = new FileOutputStream(dest)) {
+                    OutputStream os = new FileOutputStream(dest)) {
                 migrateStream(src.getName(), is, os);
             }
         } else {
@@ -213,24 +287,38 @@ public class Migration {
     }
 
 
-    private void migrateArchiveStreaming(String name, InputStream src, OutputStream dest) throws IOException {
-        try (ZipInputStream zipIs = new ZipInputStream(new CloseShieldInputStream(src));
-                ZipOutputStream zipOs = new ZipOutputStream(new CloseShieldOutputStream(dest))) {
-            ZipEntry zipEntry;
-            while ((zipEntry = zipIs.getNextEntry()) != null) {
-                String sourceName = zipEntry.getName();
-                if (isSignatureFile(sourceName)) {
-                    logger.log(Level.WARNING, sm.getString("migration.skipSignatureFile", sourceName));
+    private void migrateArchiveStreaming(InputStream src, OutputStream dest) throws IOException {
+        try (ZipArchiveInputStream srcZipStream = new ZipArchiveInputStream(CloseShieldInputStream.wrap(src));
+                ZipArchiveOutputStream destZipStream = new ZipArchiveOutputStream(CloseShieldOutputStream.wrap(dest))) {
+            ZipArchiveEntry srcZipEntry;
+            CRC32 crc32 = new CRC32();
+            while ((srcZipEntry = srcZipStream.getNextZipEntry()) != null) {
+                String srcName = srcZipEntry.getName();
+                if (isSignatureFile(srcName)) {
+                    logger.log(Level.WARNING, sm.getString("migration.skipSignatureFile", srcName));
                     continue;
                 }
-                String destName = profile.convert(sourceName);
-                JarEntry destEntry = new JarEntry(destName);
-                zipOs.putNextEntry(destEntry);
-                migrateStream(sourceName, zipIs, zipOs);
+                String destName = profile.convert(srcName);
+                if (srcZipEntry.getMethod() == ZipEntry.STORED) {
+                    ByteArrayOutputStream tempBuffer = new ByteArrayOutputStream((int) (srcZipEntry.getSize() * 1.05));
+                    migrateStream(srcName, srcZipStream, tempBuffer);
+                    crc32.update(tempBuffer.toByteArray(), 0, tempBuffer.size());
+                    MigrationZipArchiveEntry destZipEntry = new MigrationZipArchiveEntry(srcZipEntry);
+                    destZipEntry.setName(destName);
+                    destZipEntry.setSize(tempBuffer.size());
+                    destZipEntry.setCrc(crc32.getValue());
+                    destZipStream.putArchiveEntry(destZipEntry);
+                    tempBuffer.writeTo(destZipStream);
+                    destZipStream.closeArchiveEntry();
+                    crc32.reset();
+                } else {
+                    MigrationZipArchiveEntry destZipEntry = new MigrationZipArchiveEntry(srcZipEntry);
+                    destZipEntry.setName(destName);
+                    destZipStream.putArchiveEntry(destZipEntry);
+                    migrateStream(srcName, srcZipStream, destZipStream);
+                    destZipStream.closeArchiveEntry();
+                }
             }
-        } catch (ZipException ze) {
-            logger.log(Level.SEVERE, sm.getString("migration.archiveFailed", name), ze);
-            throw ze;
         }
     }
 
@@ -255,7 +343,7 @@ public class Migration {
                     continue;
                 }
                 String destName = profile.convert(srcName);
-                RenamableZipArchiveEntry destZipEntry = new RenamableZipArchiveEntry(srcZipEntry);
+                MigrationZipArchiveEntry destZipEntry = new MigrationZipArchiveEntry(srcZipEntry);
                 destZipEntry.setName(destName);
                 destZipStream.putArchiveEntry(destZipEntry);
                 migrateStream(srcName, srcZipFile.getInputStream(srcZipEntry), destZipStream);
@@ -270,8 +358,12 @@ public class Migration {
 
 
     private boolean isSignatureFile(String sourceName) {
-        return sourceName.startsWith("META-INF/")
-                && (sourceName.endsWith(".SF") || sourceName.endsWith(".RSA") || sourceName.endsWith(".DSA") || sourceName.endsWith(".EC"));
+        return sourceName.startsWith("META-INF/") && (
+                sourceName.endsWith(".SF") ||
+                sourceName.endsWith(".RSA") ||
+                sourceName.endsWith(".DSA") ||
+                sourceName.endsWith(".EC")
+                );
     }
 
 
@@ -286,20 +378,19 @@ public class Migration {
                 logger.log(Level.INFO, sm.getString("migration.archive.complete", name));
             } else {
                 logger.log(Level.INFO, sm.getString("migration.archive.stream", name));
-                migrateArchiveStreaming(name, src, dest);
+                migrateArchiveStreaming(src, dest);
                 logger.log(Level.INFO, sm.getString("migration.archive.complete", name));
             }
         } else {
             for (Converter converter : converters) {
                 if (converter.accepts(name)) {
-                    converter.convert(name, src, dest, profile);
+                    converted = converted | converter.convert(name, src, dest, profile);
                     break;
                 }
             }
         }
     }
 
-
     private boolean isArchive(String fileName) {
         return fileName.endsWith(".jar") || fileName.endsWith(".war") || fileName.endsWith(".zip");
     }
@@ -309,7 +400,7 @@ public class Migration {
         File f = new File(name);
         String filename = f.getName();
 
-        if (GlobMatcher.matchName(DEFAULT_EXCLUDES, filename, true)) {
+        if (enableDefaultExcludes && GlobMatcher.matchName(DEFAULT_EXCLUDES, filename, true)) {
             return true;
         }
 
@@ -320,9 +411,9 @@ public class Migration {
         return false;
     }
 
-    private static class RenamableZipArchiveEntry extends ZipArchiveEntry {
+    private static class MigrationZipArchiveEntry extends ZipArchiveEntry {
 
-        public RenamableZipArchiveEntry(ZipArchiveEntry entry) throws ZipException {
+        MigrationZipArchiveEntry(ZipArchiveEntry entry) throws ZipException {
             super(entry);
         }
 
