@@ -25,6 +25,7 @@ import java.util.jar.JarFile;
 
 import org.apache.commons.io.FileUtils;
 import org.junit.After;
+import org.junit.Assume;
 import org.junit.Before;
 import org.junit.Test;
 
@@ -32,14 +33,25 @@ import static org.junit.Assert.*;
 
 public class MigrationTest {
 
+    private boolean securityManagerAvailable = true;
+
     @Before
     public void setUp() {
-        System.setSecurityManager(new NoExitSecurityManager());
+        try {
+            System.setSecurityManager(new NoExitSecurityManager());
+        } catch (Throwable t) {
+            // Throws exception by default on newer Java versions
+            securityManagerAvailable = false;
+        }
     }
 
     @After
     public void tearDown() {
-        System.setSecurityManager(null);
+        try {
+            System.setSecurityManager(null);
+        } catch (Throwable t) {
+            // Throws exception by default on newer Java versions
+        }
     }
 
     @Test
@@ -57,7 +69,7 @@ public class MigrationTest {
     @Test
     public void testMigrateSingleSourceFileWithProfile() throws Exception {
         File migratedFile = new File("target/test-classes/HelloServlet.migrated.java");
-        MigrationCLI.main(new String[] {"-verbose", "-profile=EE", "target/test-classes/HelloServlet.java", migratedFile.getAbsolutePath()});
+        MigrationCLI.main(new String[] {"-logLevel=FINE", "-profile=EE", "target/test-classes/HelloServlet.java", migratedFile.getAbsolutePath()});
 
         assertTrue("Migrated file not found", migratedFile.exists());
 
@@ -83,6 +95,7 @@ public class MigrationTest {
 
     @Test
     public void testInvalidOption() throws Exception {
+        Assume.assumeTrue(securityManagerAvailable);
         File sourceFile = new File("target/test-classes/HelloServlet.java");
         File migratedFile = new File("target/test-classes/HelloServlet.migrated.java");
 
@@ -96,6 +109,7 @@ public class MigrationTest {
 
     @Test
     public void testInvalidProfile() throws Exception {
+        Assume.assumeTrue(securityManagerAvailable);
         File sourceFile = new File("target/test-classes/HelloServlet.java");
         File migratedFile = new File("target/test-classes/HelloServlet.migrated.java");
 
@@ -142,26 +156,46 @@ public class MigrationTest {
 
     @Test
     public void testMigrateJarFile() throws Exception {
+        testMigrateJarFileInternal(false);
+    }
+
+    private void testMigrateJarFileInternal(boolean zipInMemory) throws Exception {
         File jarFile = new File("target/test-classes/hellocgi.jar");
+        File jarFileTarget = new File("target/test-classes/hellocgi-target.jar");
 
         Migration migration = new Migration();
         migration.setSource(jarFile);
-        migration.setDestination(jarFile);
+        migration.setDestination(jarFileTarget);
+        migration.setZipInMemory(zipInMemory);
         migration.execute();
 
         File cgiapiFile = new File("target/test-classes/cgi-api.jar");
-        URLClassLoader classloader = new URLClassLoader(new URL[]{jarFile.toURI().toURL(), cgiapiFile.toURI().toURL()},ClassLoader.getSystemClassLoader().getParent());
+        URLClassLoader classloader = new URLClassLoader(new URL[]{jarFileTarget.toURI().toURL(), cgiapiFile.toURI().toURL()},ClassLoader.getSystemClassLoader().getParent());
 
         Class<?> cls = Class.forName("org.apache.tomcat.jakartaee.HelloCGI", true, classloader);
         assertEquals("jakarta.servlet.CommonGatewayInterface", cls.getSuperclass().getName());
 
         // check the modification of the Implementation-Version manifest attribute
-        try (JarFile jar = new JarFile(jarFile)) {
+        try (JarFile jar = new JarFile(jarFileTarget)) {
             String implementationVersion = jar.getManifest().getMainAttributes().getValue("Implementation-Version");
             assertNotNull("Missing Implementation-Version manifest attribute", implementationVersion);
             assertNotEquals("Implementation-Version manifest attribute not changed", "1.2.3", implementationVersion);
             assertTrue("Implementation-Version manifest attribute doesn't match the expected pattern", implementationVersion.matches("1\\.2\\.3-migrated-[\\d\\.]+.*"));
         }
+
+        assertTrue("hasConverted should be true", migration.hasConverted());
+    }
+
+    @Test
+    public void testMigrateJarFileInMemory() throws Exception {
+        testMigrateJarFileInternal(true);
+    }
+
+    @Test
+    public void testHasConversionsThrowsWhenNotComplete() {
+        Migration migration = new Migration();
+        IllegalStateException exception = assertThrows(IllegalStateException.class, migration::hasConverted);
+        assertEquals("Migration has not completed", exception.getMessage());
     }
 
     @Test
